@@ -724,6 +724,7 @@ class _RGFBinaryClassifier(BaseEstimator, ClassifierMixin):
         self._file_prefix = str(uuid4()) + str(_COUNTER.increment())
         _UUIDS.append(self._file_prefix)
         self._fitted = None
+        self._latest_model_loc = None
 
     def fit(self, X, y, sample_weight):
         train_x_loc = os.path.join(_TEMP_PATH, self._file_prefix + ".train.data.x")
@@ -778,11 +779,21 @@ class _RGFBinaryClassifier(BaseEstimator, ClassifierMixin):
                 print(k)
 
         self._fitted = True
+        # Find latest model location
+        model_glob = os.path.join(_TEMP_PATH, self._file_prefix + ".model*")
+        model_files = glob(model_glob)
+        if not model_files:
+            raise Exception('Model learning result is not found in {0}. '
+                            'Training is abnormally finished.'.format(_TEMP_PATH))
+        self._latest_model_loc = sorted(model_files, reverse=True)[0]
         return self
 
     def predict_proba(self, X):
         if self._fitted is None:
             raise NotFittedError(_NOT_FITTED_ERROR_DESC)
+        if not os.path.isfile(self._latest_model_loc):
+            raise Exception('Model learning result is not found in {0}. '
+                            'This is rgf_python error.'.format(_TEMP_PATH))
 
         test_x_loc = os.path.join(_TEMP_PATH, self._file_prefix + ".test.data.x")
         if sp.isspmatrix(X):
@@ -790,20 +801,12 @@ class _RGFBinaryClassifier(BaseEstimator, ClassifierMixin):
         else:
             np.savetxt(test_x_loc, X, delimiter=' ', fmt="%s")
 
-        # Find latest model location
-        model_glob = os.path.join(_TEMP_PATH, self._file_prefix + ".model*")
-        model_files = glob(model_glob)
-        if not model_files:
-            raise Exception('Model learning result is not found in {0}. '
-                            'This is rgf_python error.'.format(_TEMP_PATH))
-        latest_model_loc = sorted(model_files, reverse=True)[0]
-
         # Format test command
         pred_loc = os.path.join(_TEMP_PATH, self._file_prefix + ".predictions.txt")
         params = []
         params.append("test_x_fn=%s" % test_x_loc)
         params.append("prediction_fn=%s" % pred_loc)
-        params.append("model_fn=%s" % latest_model_loc)
+        params.append("model_fn=%s" % self._latest_model_loc)
 
         cmd = (_EXE_PATH, "predict", ",".join(params))
 
@@ -818,6 +821,20 @@ class _RGFBinaryClassifier(BaseEstimator, ClassifierMixin):
 
         return np.loadtxt(pred_loc)
 
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        if self._fitted:
+            with open(self._latest_model_loc, 'rb') as fr:
+                state["model"] = fr.read()
+        return state
+
+    def __setstate__(self, state):
+        self.__dict__.update(state)
+        if self._fitted:
+            with open(self._latest_model_loc, 'wb') as fw:
+                fw.write(self.__dict__["model"])
+            del self.__dict__["model"]
+
 
 class RGFRegressor(BaseEstimator, RegressorMixin):
     """
@@ -959,6 +976,7 @@ class RGFRegressor(BaseEstimator, RegressorMixin):
         _UUIDS.append(self._file_prefix)
         self._n_features = None
         self._fitted = None
+        self._latest_model_loc = None
 
     @property
     def n_features_(self):
@@ -1109,6 +1127,14 @@ class RGFRegressor(BaseEstimator, RegressorMixin):
                 print(k)
 
         self._fitted = True
+
+        # Find latest model location
+        model_glob = os.path.join(_TEMP_PATH, self._file_prefix + ".model*")
+        model_files = glob(model_glob)
+        if not model_files:
+            raise Exception('Model learning result is not found in {0}. '
+                            'Training is abnormally finished.'.format(_TEMP_PATH))
+        self._latest_model_loc = sorted(model_files, reverse=True)[0]
         return self
 
     def predict(self, X):
@@ -1144,20 +1170,16 @@ class RGFRegressor(BaseEstimator, RegressorMixin):
         else:
             np.savetxt(test_x_loc, X, delimiter=' ', fmt="%s")
 
-        # Find latest model location
-        model_glob = os.path.join(_TEMP_PATH, self._file_prefix + ".model*")
-        model_files = glob(model_glob)
-        if not model_files:
+        if not os.path.isfile(self._latest_model_loc):
             raise Exception('Model learning result is not found in {0}. '
                             'This is rgf_python error.'.format(_TEMP_PATH))
-        latest_model_loc = sorted(model_files, reverse=True)[0]
 
         # Format test command
         pred_loc = os.path.join(_TEMP_PATH, self._file_prefix + ".predictions.txt")
         params = []
         params.append("test_x_fn=%s" % test_x_loc)
         params.append("prediction_fn=%s" % pred_loc)
-        params.append("model_fn=%s" % latest_model_loc)
+        params.append("model_fn=%s" % self._latest_model_loc)
 
         cmd = (_EXE_PATH, "predict", ",".join(params))
 
@@ -1172,3 +1194,17 @@ class RGFRegressor(BaseEstimator, RegressorMixin):
 
         y_pred = np.loadtxt(pred_loc)
         return y_pred
+
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        if self._fitted:
+            with open(self._latest_model_loc, 'rb') as fr:
+                state["model"] = fr.read()
+        return state
+
+    def __setstate__(self, state):
+        self.__dict__.update(state)
+        if self._fitted:
+            with open(self._latest_model_loc, 'wb') as fw:
+                fw.write(self.__dict__["model"])
+            del self.__dict__["model"]
